#!/bin/bash

# Simple Secure Text Interface
#

usage_exit() {
  if (($1)) ; then echo "$0: No dictionary found."; fi
  echo "Usage: $0  [-q] [-a]|[-e]|[-E]|[-c] [-l]|[-d directory] [string]" 1>&2
 cat << USAGE

 mode and options :

    lookup (-a): ファイル中のstringを探して、ファイル名を表示
      表示後、対話的に次のモードへ
      decode : デコードする
      remove : 削除する。要確認
      export : デコードして、exportedディレクトリに出力
    encode (-e): ディレクトリ内のファイルをエンコードする
    force encode (-E): ディレクトリ内のファイルをエンコードする。上書き
    cleanup (-c): sourceディレクトリ内を削除する
    fastlookup (default): 1行目を検索 (エンコード時にファイル名は、テキストの1行目に)

    ディレクトリベースのデフォルトは、$HOME/Dropbox/sstdb/
        -l: ディレクトリベースをローカル($HOME/.sstdb/)にする。
        -d directory: ディレクトリベースをdirectoryにする。

USAGE
  exit 1
}


debug () { # debug出力を抑止したいなら、以下のtrueをfaluseに
    true && message "$*"
}

message () {
    [ "$quiet" = "true" ] && return
    if [ "$1" = "-n" ]; then
        shift
        echo -n "$*"  
    else
        echo "$*"
    fi
}

##
enc_action() {
    src="$*"
    message -n "${src}"

    # ファイル名は、1行目から全角文字をできるだけとりのぞいて、ファイル名に使えない/は.に
    # tr処理は、enc_dec.awkと合わせること : Ubuntuのtrは、日本語文字を上手く扱えない
    #dst=$(head -n 1 "${src}" | tr 'Ａ-Ｚａ-ｚ０-９/' 'A-Za-z0-9.').txt
	# なのでsedを使ったもの
    dst=$(head -n 1 "${src}" | sed 'y;ＡＢＣＤＥＦＧＨＩＪＫＬＭＮＯＰＱＲＳＴＵＶＷＸＹＺａｂｃｄｅｆｇｈｉｊｋｌｍｎｏｐｑｒｓｔｕｖｗｘｙｚ０１２３４５６７８９/;ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789.;').txt 

    if [ "${dst}" == ".txt" ]; then # 1. assetsに${dst}が存在しているならスキップ
        message " 1st line is empty."
        return
    fi
    if [ "${mode}" == "encode" -a -e "${assets}/${dst}" ]; then # 1. assetsに${dst}が存在しているならスキップ
        message " already exists."
        return
    fi

	if [ "${assets}/${dst}" -nt "${src}" ]; then  # ソースが更新されていないならスキップ
        message " no updates."
        return
    fi

    # ~~to be enc~~を見つけて暗号化
    awk -f "${enc_dec}" -v mode="enc" -v pass="${pp}" "${src}" > "${assets}/${dst}"
    if [ $? -eq 0 ]; then
        message " encrypted, and added."
    else
        message " encrypt failed."
    fi
}


each_dir() {
  while [ "$1" != "" ]
  do
    s="$1"
    if [ -d "$s" ]; then
      pushd "$s" > /dev/null
      each_dir *
      popd > /dev/null
    else 
      enc_action "$s"
    fi
    shift
  done
}


encode() {
    if [ ! -d ${src} ]; then
      message "${src} is not a directory, quitting."
      exit 2
    fi

    read -sp "password:" pp
	echo
    pushd "${src}" > /dev/null
    each_dir *
    popd > /dev/null
}


decode() {
    read -sp "password:" pp
    awk -f "${enc_dec}" -v mode="dec" -v pass="$pp" "$*" | less
}


export_action() {
    if [ ! -d "${exported}" ]; then
        message "${exported} dose not exist."
        return
    fi
    awk -f "${enc_dec}" -v mode="dec" "${assets}/$1" > "${exported}/$1"
    echo
}


cleanup() {
    local q
    read -p "cleanup the source directory? (y/n)" q
    case $q in
      [yY]* ) message "cleaning up!"; rm -rf "${src}/*"; ;;
      * ) message "do nothing." ;;
    esac
}


remove() {
    local q
    read -p "Remove \"${1}\"? (y/n)" q
    case $q in
      [yY]* ) rm -f "${1}" && message "removed!"; ;;
      * ) message "do nothing." ;;
    esac
}


lookup() {
    list=()
    local f
    local q
    local str="$*"

    if [ "${str}" != "" ] ; then
        if [ "$mode" = "fastlookup" ]; then
            message "[fast lookup] ${str}"
            execute="awk -v IGNORECASE=1 \"/${str}/{print FILENAME} {nextfile}\" \"${assets}/\"*"
        else
            message "[lookup] ${str}"
            execute="fgrep -il \"${str}\" \"${assets}/\"*"
        fi
    else
        message "[list up]"
        execute="ls -1 \"${assets}/\"*"
    fi

	while read -r f ; do
		f=$(basename "${f}")
		list+=("$f")
	done < <(eval ${execute})

	echo ":: found ${#list[@]} files ::"
	if [ "${str}" != "" ] ; then
		for f in "${list[@]}"; do
			echo
			echo -n "\"${f%.txt}\" : "
			read -p "(Show/Decode/Export/Remove/Next/Quit)? " q
			echo
			case ${q} in
			  [sS]* ) less "${assets}/${f}"; ;;
			  [dD]* ) decode "${assets}/${f}"; ;;
			  [eE]* ) export_action "${f}"; ;;
			  [rR]* ) remove "${assets}/${f}"; ;;
			  [qQ]* ) return; ;;
			  [nN]* ) ;;
			  * ) ;;
			esac
		done
	else
		read -n1 -p "(show/All/Quit)? " q
		case ${q} in
			[aA]* ) q="a"; ;;
			[qQ]* ) return; ;;
			* ) ;;
		esac
		lines=$( stty -a | awk -F ";" 'NR==1{for (i=1; i<=NF; i++) print $i}' | awk '/rows/{ sub("rows",""); sub(";",""); print }' )
		for ((i=0; i< ${#list[@]}; i++)); do
			echo "${list[$i]%.txt}"
			if [ "$q" != "a" ]; then
				if (( (i + 1) % lines == 0 )); then
				  read -n1 -p "(next/All/Quit)? " q
					case ${q} in
						[aA]* ) q="a"; ;;
						[qQ]* ) return; ;;
						* ) ;;
					esac
				fi
			fi
		done
	fi
}

## Main function

enc_dec=$(cd $(dirname $0); pwd)"/enc_dec.awk" # このスクリプトと同じパスにあること


localdir="${HOME}/.sstdb"
src="${localdir}/source" # ソースディレクトリのデフォルトは、ネットドライブを指定させない
exported="${localdir}/exported"
quiet="false"

# Dropboxをデフォルトに
dirbase="${WINHOME:-${HOME}}/Dropbox/sstdb"

if [ ! -d ${dirbase} ]; then
	dirbase=${localdir}
	message "Dropboxではなく、${dirbase}を使います。"
fi

mode="fastlookup"
while getopts d:lqaceE? OPT
do
  case $OPT in
    d) dirbase="${OPTARG}"; ;;
    l) dirbase="${localdir}"; ;;
    q) quiet="true"; ;;
    a) mode="lookup"; ;;
    c) mode="cleanup"; ;;
    e) mode="encode"; ;;
    E) mode="forceencode"; ;;
    \?) usage_exit ;;
  esac
done
shift $((OPTIND - 1))

str="$*"

assets="${dirbase}/assets"

message "mode=${mode}"
message "----"


case ${mode} in
    "lookup" | "fastlookup")
        if [ ! -d "${assets}" ]; then
            message "assets=${assets}"  " is not exist."
            exit 1
        fi
        lookup "${str}";;

    "encode" | "forceencode")
        if [ ! -d "${assets}" ]; then
            message "assets=${assets}" " is not exist."
            mode="exit"
        fi
        if [ ! -d "${src}" ]; then
            message "source=${src}" " is not exist."
            mode="exit"
        fi
        [ "$mode" = "exit" ] && exit 1
        encode ;;

    "cleanup")
        if [ ! -d "${src}" ]; then
            message "source=${src}" " is not exist."
            exit 1
        fi
        cleanup ;;
esac

echo

